"""
Class definition for a data source
"""

from pydantic import BaseModel
from typing import List, Dict, Optional
from ...sources import (FORMAT_BASIC_INFO,
                        FORMAT_BASIC_INFO_WITH_METADATA,
                        FORMAT_IDENTIFIER_ONLY,
                        FORMAT_FULL_RECORD)

# pdoc init
__pdoc__: Dict = {}


class DataSource(BaseModel):
    """
    Data source object

    Attributes:
        identifier: the unique AuroraX ID for this data source
        program: the program for this data source
        platform: the platform for this data source
        instrument_type: the instrument type for this data source
        source_type: the data source type for this data source. Options are
            in the pyaurorax.sources module, or at the top level using the
            pyaurorax.SOURCE_TYPE_* variables.
        display_name: the display name for this data source
        metadata: metadata for this data source (arbitrary keys and values)
        owner: the owner's email address of this data source
        maintainers: the email addresses of AuroraX accounts that can alter
            this data source and its associated records
        ephemeris_metadata_schema: a list of dictionaries capturing the metadata
            keys and values that can appear in ephemeris records associated with
            this data source
        data_product_metadata_schema: a list of dictionaries capturing the metadata
            keys and values that can appear in data product records associated with
            this data source
        format: the format used when printing the data source, defaults to
            "full_record". Other options are in the pyaurorax.sources module, or
            at the top level using the pyaurorax.FORMAT_* variables.
    """
    identifier: Optional[int] = None
    program: Optional[str] = None
    platform: Optional[str] = None
    instrument_type: Optional[str] = None
    source_type: Optional[str] = None
    display_name: Optional[str] = None
    metadata: Optional[Dict] = None
    owner: Optional[str] = None
    maintainers: Optional[List[str]] = None
    ephemeris_metadata_schema: Optional[List[Dict]] = None
    data_product_metadata_schema: Optional[List[Dict]] = None
    format: Optional[str] = FORMAT_FULL_RECORD

    def __str__(self) -> str:
        """
        String method

        Returns:
            string format of DataSource object
        """
        return self.__repr__()

    def __repr__(self) -> str:
        """
        Object representation

        Returns:
            object representation of DataSource object
        """
        # set each attribute string
        attr_identifier = "None" if self.identifier is None else f"{self.identifier}"
        attr_program = "None" if self.program is None else f"'{self.program}'"
        attr_platform = "None" if self.platform is None else f"'{self.platform}'"
        attr_instrument_type = "None" if self.instrument_type is None else f"'{self.instrument_type}'"
        attr_source_type = "None" if self.source_type is None else f"'{self.source_type}'"
        attr_display_name = "None" if self.display_name is None else f"'{self.display_name}'"
        attr_metadata = "None" if self.metadata is None else f"{self.metadata}"
        attr_owner = "None" if self.owner is None else f"'{self.owner}'"
        attr_maintainers = "None" if self.maintainers is None else f"{self.maintainers}"
        attr_eph_schema = "None" if self.ephemeris_metadata_schema is None else f"{self.ephemeris_metadata_schema}"
        attr_dp_schema = "None" if self.data_product_metadata_schema is None else f"{self.data_product_metadata_schema}"

        # shorten strings
        max_len = 20
        if (len(attr_metadata) > max_len):
            attr_metadata = attr_metadata[0:max_len] + "...}"
        if (len(attr_eph_schema) > max_len):
            attr_eph_schema = attr_eph_schema[0:max_len] + "...}]"
        if (len(attr_dp_schema) > max_len):
            attr_dp_schema = attr_dp_schema[0:max_len] + "...}]"

        # for each format type, construct the repr to return
        if (self.format == FORMAT_IDENTIFIER_ONLY):
            r = "DataSource(identifier=%s)" % (attr_identifier)
        elif (self.format == FORMAT_BASIC_INFO):
            r = "DataSource(identifier=%s, program=%s, platform=%s, " \
                "instrument_type=%s, source_type=%s, display_name=%s)" % (attr_identifier,
                                                                          attr_program,
                                                                          attr_platform,
                                                                          attr_instrument_type,
                                                                          attr_source_type,
                                                                          attr_display_name)
        elif (self.format == FORMAT_BASIC_INFO_WITH_METADATA):
            r = "DataSource(identifier=%s, program=%s, platform=%s, " \
                "instrument_type=%s, source_type=%s, display_name=%s, " \
                "metadata=%s)" % (attr_identifier,
                                  attr_program,
                                  attr_platform,
                                  attr_instrument_type,
                                  attr_source_type,
                                  attr_display_name,
                                  attr_metadata)
        elif (self.format == FORMAT_FULL_RECORD):
            r = "DataSource(identifier=%s, program=%s, platform=%s, " \
                "instrument_type=%s, source_type=%s, display_name=%s, " \
                "metadata=%s, owner=%s, maintainers=%s, ephemeris_metadata_schema=%s, " \
                "data_product_metadata_schema=%s)" % (attr_identifier,
                                                      attr_program,
                                                      attr_platform,
                                                      attr_instrument_type,
                                                      attr_source_type,
                                                      attr_display_name,
                                                      attr_metadata,
                                                      attr_owner,
                                                      attr_maintainers,
                                                      attr_eph_schema,
                                                      attr_dp_schema)

        # return constructed repr
        return r



"""
Functions for retrieving availablity information
"""

import datetime
from typing import Dict, List, Optional
from .classes.availability_result import AvailabilityResult
from ..sources import FORMAT_DEFAULT, DataSource
from ..api import urls, AuroraXRequest

# pdoc init
__pdoc__: Dict = {}


def ephemeris(start: datetime.date,
              end: datetime.date,
              program: Optional[str] = None,
              platform: Optional[str] = None,
              instrument_type: Optional[str] = None,
              source_type: Optional[str] = None,
              owner: Optional[str] = None,
              format: Optional[str] = FORMAT_DEFAULT,
              slow: Optional[bool] = False) -> List[AvailabilityResult]:
    """
    Retrieve information about the number of existing ephemeris records

    Args:
        start: start date to retrieve availability info from (inclusive)
        end: end date to retrieve availability info until (inclusive)
        program: program name to filter sources by, defaults to None
        platform: platform name to filter sources by, defaults to None
        instrument_type: instrument type to filter sources by, defaults to None
        source_type: source type to filter sources by, defaults to None. Other
            options are in the pyaurorax.sources module, or at the top level
            using the pyaurorax.SOURCE_TYPE_* variables.
        owner: owner email address to filter sources by, defaults to None
        format: the format of the data sources returned, defaults to "basic_info".
            Other options are in the pyaurorax.sources module, or at the top level
            using the pyaurorax.FORMAT_* variables.
        slow: query the data using a slower, but more accurate method, defaults to False

    Returns:
        ephemeris availability information matching the requested parameters
    """
    # set parameters
    params = {
        "start": start.strftime("%Y-%m-%d"),
        "end": end.strftime("%Y-%m-%d"),
        "program": program,
        "platform": platform,
        "instrument_type": instrument_type,
        "source_type": source_type,
        "owner": owner,
        "format": format,
        "slow": slow,
    }

    # do request
    req = AuroraXRequest(method="get",
                         url=urls.ephemeris_availability_url,
                         params=params)
    res = req.execute()

    # cast data source record
    for i in range(0, len(res.data)):
        ds = DataSource(**res.data[i]["data_source"], format=format)
        res.data[i]["data_source"] = ds

    # return
    return [AvailabilityResult(**av) for av in res.data]


def data_products(start: datetime.date,
                  end: datetime.date,
                  program: Optional[str] = None,
                  platform: Optional[str] = None,
                  instrument_type: Optional[str] = None,
                  source_type: Optional[str] = None,
                  owner: Optional[str] = None,
                  format: Optional[str] = FORMAT_DEFAULT,
                  slow: Optional[bool] = False) -> List[AvailabilityResult]:
    """
    Retrieve information about the number of existing data product records

    Args:
        start: start date to retrieve availability info from (inclusive)
        end: end date to retrieve availability info until (inclusive)
        program: program name to filter sources by, defaults to None
        platform: platform name to filter sources by, defaults to None
        instrument_type: instrument type to filter sources by, defaults to None
        source_type: source type to filter sources by, defaults to None. Other
            options are in the pyaurorax.sources module, or at the top level
            using the pyaurorax.SOURCE_TYPE_* variables.
        owner: owner email address to filter sources by, defaults to None
        format: the format of the data sources returned, defaults to "basic_info".
            Other options are in the pyaurorax.sources module, or at the top level
            using the pyaurorax.FORMAT_* variables.
        slow: query the data using a slower, but more accurate method, defaults to False

    Returns:
        data product availability information matching the requested parameters
    """
    # set parameters
    params = {
        "start": start.strftime("%Y-%m-%d"),
        "end": end.strftime("%Y-%m-%d"),
        "program": program,
        "platform": platform,
        "instrument_type": instrument_type,
        "source_type": source_type,
        "owner": owner,
        "format": format,
        "slow": slow,
    }

    # do request
    req = AuroraXRequest(method="get",
                         url=urls.data_products_availability_url,
                         params=params)
    res = req.execute()

    # cast data source record
    for i in range(0, len(res.data)):
        ds = DataSource(**res.data[i]["data_source"], format=format)
        res.data[i]["data_source"] = ds

    # return
    return [AvailabilityResult(**av) for av in res.data]



"""
Class definition used for containing Availability information
"""

from pydantic import BaseModel
from typing import Dict, Optional
from ...sources import DataSource

# pdoc init
__pdoc__: Dict = {}


class AvailabilityResult(BaseModel):
    """
    Availability information object

    Attributes:
        data_source: the data source that the records are associated with
        available_data_products: the data product availability information
        available_ephemeris: the ephemeris availability information
    """
    data_source: DataSource
    available_data_products: Optional[Dict[str, int]] = None
    available_ephemeris: Optional[Dict[str, int]] = None

    def __str__(self) -> str:
        """
        String method

        Returns:
            string format of AvailabilityResult
        """
        return self.__repr__()



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['pyaurorax',
 'pyaurorax._internal',
 'pyaurorax.api',
 'pyaurorax.api.classes',
 'pyaurorax.availability',
 'pyaurorax.availability.classes',
 'pyaurorax.cli',
 'pyaurorax.cli.availability',
 'pyaurorax.cli.conjunctions',
 'pyaurorax.cli.data_products',
 'pyaurorax.cli.ephemeris',
 'pyaurorax.cli.sources',
 'pyaurorax.cli.util',
 'pyaurorax.conjunctions',
 'pyaurorax.conjunctions.classes',
 'pyaurorax.conjunctions.swarmaurora',
 'pyaurorax.data_products',
 'pyaurorax.data_products.classes',
 'pyaurorax.ephemeris',
 'pyaurorax.ephemeris.classes',
 'pyaurorax.metadata',
 'pyaurorax.requests',
 'pyaurorax.sources',
 'pyaurorax.sources.classes',
 'pyaurorax.util']

package_data = \
{'': ['*']}

install_requires = \
['click>=8.0.3,<9.0.0',
 'humanize>=3.2.0,<4.0.0',
 'pydantic>=1.7.3,<2.0.0',
 'python-dateutil>=2.8.2,<3.0.0',
 'requests>=2.25.1,<3.0.0',
 'termcolor>=1.1.0,<2.0.0',
 'texttable>=1.6.4,<2.0.0']

extras_require = \
{'aacgmv2': ['aacgmv2>=2.6.2,<3.0.0']}

entry_points = \
{'console_scripts': ['aurorax-cli = pyaurorax.cli.cli:cli']}

setup_kwargs = {
    'name': 'pyaurorax',
    'version': '0.10.0',
    'description': 'Python library for interacting with the AuroraX API',
    'long_description': '<a href="https://aurorax.space/"><img alt="AuroraX" src="logo.svg" height="60"></a>\n\n[![GitHub tests](https://github.com/aurorax-space/pyaurorax/actions/workflows/test_standard.yml/badge.svg)](https://github.com/aurorax-space/pyaurorax/actions/workflows/test_standard.yml)\n[![PyPI version](https://img.shields.io/pypi/v/pyaurorax.svg)](https://pypi.python.org/pypi/pyaurorax/)\n[![MIT license](https://img.shields.io/badge/License-MIT-blue.svg)](https://github.com/aurorax-space/pyaurorax/blob/main/LICENSE)\n[![PyPI Python versions](https://img.shields.io/pypi/pyversions/pyaurorax.svg)](https://pypi.python.org/pypi/pyaurorax/)\n[![DOI](https://zenodo.org/badge/DOI/10.5281/zenodo.6098075.svg)](https://doi.org/10.5281/zenodo.6098075)\n\nPyAuroraX is a Python library for interacting with [AuroraX](https://aurorax.space), a project working to be the world\'s first and foremost data platform for auroral science. The primary objective of AuroraX is to enable mining and exploration of existing and future auroral data, enabling key science and enhancing the benefits of the world\'s investment in auroral instrumentation. This will be accomplished with the development of key systems/standards for uniform metadata generation and search, image content analysis, interfaces to leading international tools, and a community involvement that includes more than 80% of the world\'s data providers.\n\nPyAuroraX officially supports Python 3.6, 3.7, 3.8, and 3.9 (Python 3.10 not currently supported).\n\nSome links to help:\n- [AuroraX main website](https://aurorax.space)\n- [PyAuroraX documentation](https://docs.aurorax.space/code/overview)\n- [PyAuroraX API Reference](https://docs.aurorax.space/code/pyaurorax_api_reference/pyaurorax)\n\n## Installation\n\nPyAuroraX is available on PyPI so pip can be used to install it:\n\n```console\n$ pip install pyaurorax\n```\n\nTo get full functionality, you can install PyAuroraX with the aacgmv2 dependency. Note that without this, the calculate_btrace methods in the util module will show warning messages. All other functionality will work without this dependency.\n\n```console\n$ pip install pyaurorax[aacgmv2]\n```\n\nFuthermore, if you want the most bleeding edge version of PyAuroraX, you can install it directly from the Github repository:\n\n```console\n$ git clone https://github.com/aurorax-space/pyaurorax.git\n$ cd pyaurorax\n$ pip install .\n[ or with the aacgmv2 extra ]\n$ pip install .[aacgmv2]\n```\n\n## Usage\n\nThere are two things you can use as part of the PyAuroraX library: the main library, and the command line tool.\n\n### Library import\n\nYou can import the library using the following statement:\n\n```python\n>>> import pyaurorax\n```\n\n### CLI program\n\nThe program `aurorax-cli` is included in the PyAuroraX package as a command line tool. Try it out using:\n\n```\n$ aurorax-cli --help\n```\n\n## Development\n\nSome common things you can do:\n- `make update` Update the Python dependency libraries\n- `tools/bump_version.py` Bump the version number\n- `make test-pytest-unauthorized-access` Only run the authorization tests\n- `make test-pytest-read` Only run the read-based tests\n- `make test-pytest-create-update-delete` Only run the write-based tests\n- `make docs` Generate pdoc documentation\n\n### Setup\n\nClone the repository and install primary and development dependencies using Poetry.\n\n```console\n$ git clone git@github.com:aurorax-space/pyaurorax.git\n$ cd pyaurorax\n$ make install\n```\n\n### Documentation\n\nDocumentation for the PyAuroraX project is managed by a separate repository [here](https://github.com/aurorax-space/docs). However, you are still able to generate the documentation for this repo for testing/development purposes. To generate the docs, run the following:\n\n```console\n$ make docs\n```\n\n### Testing\n\nPyAuroraX includes several test evaluations bundled into two groups: linting and functionality tests. The linting includes looking through the codebase using tools such as Flake8, PyLint, Pycodestyle, Bandit, and MyPy. The functionality tests use PyTest to test modules in the library.\n\nWhen running the functionality tests using PyTest, you must have the environment variable `AURORAX_APIKEY_STAGING` set to your API key on the staging API system. Alternatively, you can specifiy your API key using the command line (see example at the bottom of this section).\n\nThere exist several makefile targets to help run these tests quicker/easier. Below are the available commands:\n\n- `make test-linting` Run all linting tests\n- `make test-pytest` Run all automated functional tests\n- `make test-flake8` Run Flake8 styling tests\n- `make test-pylint` Run PyLint styling tests\n- `make test-pycodestyle` Run pycodestyle styling tests\n- `make test-bandit` Run Bandit security test\n- `make test-mypy` Run mypy type checking test\n- `make test-coverage` View test coverage report (must be done after `make test-pytest` or other coverage command)\n\nThe PyTest functionality tests include several categories of tests. You can run each category separately if you want using the "markers" feature of PyTest. All markers are found in the pytest.ini file at the root of the repository.\n\n- `poetry run pytest --markers` List all markers\n- `poetry run pytest -v -m accounts` Perform only the tests for the "accounts" marker\n- `poetry run pytest -v -m availability` Perform only the tests for the "availability" marker\n- `poetry run pytest -v -m conjunctions` Perform only the tests for the "conjunctions" marker\n- `poetry run pytest -v -m ephemeris` Perform only the tests for the "ephemeris" marker\n- `poetry run pytest -v -m exceptions` Perform only the tests for the "exceptions" marker\n- `poetry run pytest -v -m location` Perform only the tests for the "location" marker\n- `poetry run pytest -v -m metadata` Perform only the tests for the "metadata" marker\n- `poetry run pytest -v -m requests` Perform only the tests for the "request" marker\n- `poetry run pytest -v -m sources` Perform only the tests for the "sources" marker\n- `poetry run pytest -v -m util` Perform only the tests for the "util" marker\n\nBelow are some more commands for advanced usages of PyTest.\n\n- `poetry run pytest -v` Run all tests in verbose mode\n- `poetry run pytest --collect-only` List all available tests\n- `poetry run pytest --markers` List all markers (includes builtin, plugin and per-project ones)\n- `cat pytest.ini` List custom markers\n- `poetry run pytest tests/test_suite/ephemeris/test_ephemeris.py::test_get_request_url -v` Run a single specific test\n\nYou can also run Pytest against a different API. By default, it runs agains the staging API, but you can alternatively tell it to run against the production API, or a local instance.\n\n- `poetry run pytest -v --env=production` Run all tests against production API, using the AURORAX_APIKEY_PRODUCTION environment variable\n- `poetry run pytest --env=local --host=http://localhost:3000` Run all tests against a local instance of the API, using the AURORAX_APIKEY_LOCAL environment variable\n- `poetry run pytest -v --api-key=SOME_API_KEY` Run all tests with the specified API key (will run against the staging API since that\'s the default)\n- `poetry run pytest --help` View usage for pytest, including the usage for custom options (see the \'custom options\' section of the output)\n\nBelow are some more commands for evaluating the PyTest coverage.\n\n- `poetry run coverage report` View test coverage report\n- `poetry run coverage html` Generate an HTML page of the coverage report\n- `poetry run coverage report --show-missing` View the test coverage report and include the lines deemed to be not covered by tests\n\nNote that the coverage report only gets updated when using the Makefile pytest targets, or when running coverage manually like `coverage run -m pytest -v`. More information about usage of the `coverage` command can be found [here](https://coverage.readthedocs.io).\n',
    'author': 'Maryam Sohrabi',
    'author_email': 'sohrabm@ucalgary.ca',
    'maintainer': 'Maryam Sohrabi',
    'maintainer_email': 'sohrabm@ucalgary.ca',
    'url': 'https://github.com/aurorax-space/pyaurorax',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'extras_require': extras_require,
    'entry_points': entry_points,
    'python_requires': '>=3.6,<3.10',
}


setup(**setup_kwargs)



"""
Class definition used for managing the response from an
API request
"""

from pydantic import BaseModel
from typing import Dict, Any

# pdoc init
__pdoc__: Dict = {}


class AuroraXResponse(BaseModel):
    """
    AuroraX API response class

    Attributes:
        request: the request object
        data: the data received as part of the request
        status_code: the HTTP status code received when making the request
    """
    request: Any
    data: Any
    status_code: int

    def __str__(self) -> str:
        """
        String method

        Returns:
            string format of AuroraXResponse
        """
        return self.__repr__()

    def __repr__(self) -> str:
        """
        Object representation

        Returns:
            object representation of AuroraXResponse
        """
        return f"AuroraXResponse [{self.status_code}]"



"""
This module contains the templates for different search
requests (conjunction, data products, ephemeris)
"""

CONJUNCTION_SEARCH_TEMPLATE = {
    "start": "2020-01-01T00:00:00",
    "end": "2020-01-01T23:59:59",
    "ground": [
        {
            "programs": [
                "string"
            ],
            "platforms": [
                "string"
            ],
            "instrument_types": [
                "string"
            ],
            "ephemeris_metadata_filters": {
                "logical_operator": "AND",
                "expressions": [
                    {
                        "key": "string",
                        "operator": "=",
                        "values": [
                            "string"
                        ]
                    }
                ]
            }
        }
    ],
    "space": [
        {
            "programs": [
                "string"
            ],
            "platforms": [
                "string"
            ],
            "instrument_types": [
                "string"
            ],
            "hemisphere": [
                "northern"
            ],
            "ephemeris_metadata_filters": {
                "logical_operator": "AND",
                "expressions": [
                    {
                        "key": "string",
                        "operator": "=",
                        "values": [
                            "string"
                        ]
                    }
                ]
            }
        }
    ],
    "events": [
        {
            "programs": [
                "string"
            ],
            "platforms": [
                "string"
            ],
            "instrument_types": [
                "string"
            ],
            "ephemeris_metadata_filters": {
                "logical_operator": "AND",
                "expressions": [
                    {
                        "key": "string",
                        "operator": "=",
                        "values": [
                            "string"
                        ]
                    }
                ]
            }
        }
    ],
    "conjunction_types": [
        "nbtrace"
    ],
    "max_distances": {
        "ground1-space1": 300,
        "ground2-space1": 400,
        "ground1-events1": 900,
        "space1-space2": None
    }
}


DATA_PRODUCTS_SEARCH_TEMPLATE = {
    "start": "2020-01-01T00:00:00",
    "end": "2020-01-01T23:59:59",
    "data_sources": {
        "programs": [
            "string"
        ],
        "platforms": [
            "string"
        ],
        "instrument_types": [
            "string"
        ],
        "data_product_metadata_filters": {
            "logical_operator": "AND",
            "expressions": [
                {
                    "key": "string",
                    "operator": "=",
                    "values": [
                        "string"
                    ]
                }
            ]
        }
    },
    "data_product_type_filters": [
        "keogram"
    ]
}


EPHEMERIS_SEARCH_TEMPLATE = {
    "start": "2020-01-01T00:00:00",
    "end": "2020-01-01T23:59:59",
    "data_sources": {
        "programs": [
            "string"
        ],
        "platforms": [
            "string"
        ],
        "instrument_types": [
            "string"
        ],
        "ephemeris_metadata_filters": {
            "logical_operator": "AND",
            "expressions": [
                {
                    "key": "string",
                    "operator": "=",
                    "values": [
                        "string"
                    ]
                }
            ]
        }
    },
}



"""
Separted classes and functions used by the data_products module.

Note that these classes and variables are all imported higher up at the
top of the data_products module. They can be referenced from there instead
of digging in deeper to these submodules.
"""



import sys
import click
import pprint
import json
import os
import datetime
import pyaurorax
from dateutil.parser import parse
from ..helpers import (print_request_logs_table,
                       print_request_status,
                       get_search_data)
from ..templates import DATA_PRODUCTS_SEARCH_TEMPLATE


def __create_search_object_from_query(q):
    start = parse(q["start"], ignoretz=True)
    end = parse(q["end"], ignoretz=True)
    data_product_types = None if "data_product_type_filters" not in q else q["data_product_type_filters"]
    programs = None if "programs" not in q["data_sources"] else q["data_sources"]["programs"]
    platforms = None if "platforms" not in q["data_sources"] else q["data_sources"]["platforms"]
    instrument_types = None if "instrument_types" not in q["data_sources"] else q["data_sources"]["instrument_types"]
    metadata_filters = None
    metadata_filters_logical_operator = None
    if ("data_product_metadata_filters" in q["data_sources"]):
        if ("expressions" in q["data_sources"]["data_product_metadata_filters"]):
            metadata_filters = q["data_sources"]["data_product_metadata_filters"]["expressions"]
        if ("logical_operator" in q["data_sources"]["data_product_metadata_filters"]):
            metadata_filters_logical_operator = q["data_sources"]["data_product_metadata_filters"]["logical_operator"]
    s = pyaurorax.data_products.Search(start,
                                       end,
                                       programs=programs,
                                       platforms=platforms,
                                       instrument_types=instrument_types,
                                       data_product_types=data_product_types,
                                       metadata_filters=metadata_filters,
                                       metadata_filters_logical_operator=metadata_filters_logical_operator)
    return s


@click.group("data_products", help="Interact with data product searches")
def data_products_group():
    pass


@data_products_group.command("get_status",
                             short_help="Get status info for a data product search request")
@click.argument("request_uuid", type=str)
@click.option("--show-logs", "show_logs", is_flag=True,
              help="Show the logs for the request")
@click.option("--show-query", "show_query", is_flag=True,
              help="Show the query for the request")
@click.option("--filter-logs",
              type=click.Choice(["debug", "info", "warn", "error"]),
              help="Filter log messages (used with --show-logs)")
@click.option("--table-max-width", "--max-width", type=int,
              help="Max width for the logs table")
@click.pass_obj
def get_status(config, request_uuid, show_logs, show_query, filter_logs, table_max_width):
    """
    Get information for a data product search request

    \b
    REQUEST_UUID    the request unique identifier
    """
    # get request status
    try:
        url = pyaurorax.api.urls.data_products_request_url.format(request_uuid)
        s = pyaurorax.requests.get_status(url)
    except pyaurorax.AuroraXNotFoundException as e:
        click.echo("%s occurred: request ID not found" % (type(e).__name__))
        sys.exit(1)
    except pyaurorax.AuroraXException as e:
        click.echo("%s occurred: %s" % (type(e).__name__, e.args[0]))
        sys.exit(1)

    # print status nicely
    print_request_status(s,
                         show_logs=show_logs,
                         show_query=show_query,
                         filter_logs=filter_logs,
                         table_max_width=table_max_width)


@data_products_group.command("get_logs",
                             short_help="Get logs for a data product search request")
@click.argument("request_uuid", type=str)
@click.option("--filter", "--filter-logs", "filter_",
              type=click.Choice(["debug", "info", "warn", "error"]),
              help="Filter log messages")
@click.option("--table-max-width", "--max-width", type=int,
              help="Max width for the logs table")
@click.pass_obj
def get_logs(config, request_uuid, filter_, table_max_width):
    """
    Get the logs for a data product search request

    \b
    REQUEST_UUID    the request unique identifier
    """
    # get request status
    try:
        url = pyaurorax.api.urls.data_products_request_url.format(request_uuid)
        s = pyaurorax.requests.get_status(url)
    except pyaurorax.AuroraXNotFoundException as e:
        click.echo("%s occurred: request ID not found" % (type(e).__name__))
        sys.exit(1)
    except pyaurorax.AuroraXException as e:
        click.echo("%s occurred: %s" % (type(e).__name__, e.args[0]))
        sys.exit(1)

    # print out the logs nicely
    if ("logs" in s):
        print_request_logs_table(s["logs"],
                                 filter_level=filter_,
                                 table_max_width=table_max_width)
    else:
        click.echo("Search logs: missing, unable to display")


@data_products_group.command("get_query",
                             short_help="Get query for a data product search request")
@click.argument("request_uuid", type=str)
@click.pass_obj
def get_query(config, request_uuid):
    """
    Get the query for a data product search request

    \b
    REQUEST_UUID    the request unique identifier
    """
    # get request status
    try:
        url = pyaurorax.api.urls.data_products_request_url.format(request_uuid)
        s = pyaurorax.requests.get_status(url)
    except pyaurorax.AuroraXNotFoundException as e:
        click.echo("%s occurred: request ID not found" % (type(e).__name__))
        sys.exit(1)
    except pyaurorax.AuroraXException as e:
        click.echo("%s occurred: %s" % (type(e).__name__, e.args[0]))
        sys.exit(1)

    # print out query
    if ("query" in s["search_request"]):
        query_to_show = s["search_request"]["query"]
        del query_to_show["request_id"]
        click.echo(pprint.pformat(query_to_show))
    else:
        click.echo("\nSearch query missing from request status, unable to display")


@data_products_group.command("get_data",
                             short_help="Get data for a data product search request")
@click.argument("request_uuid", type=str)
@click.option("--outfile", type=str, help="Output file to save data to (a .json file)")
@click.option("--output-to-terminal", type=click.Choice(["dict", "objects"]),
              help="Output data to terminal in a certain format (instead of to file)")
@click.option("--indent", type=int, default=2, show_default=True,
              help="Indentation when saving data to file")
@click.option("--minify", is_flag=True, help="Minify the JSON data saved to file")
@click.pass_obj
def get_data(config, request_uuid, outfile, output_to_terminal, indent, minify):
    """
    Get the data for a data product search request

    \b
    REQUEST_UUID    the request unique identifier
    """
    get_search_data("data_products",
                    request_uuid,
                    outfile,
                    output_to_terminal,
                    indent,
                    minify)


@data_products_group.command("search_resubmit",
                             short_help="Resubmit a data product search request")
@click.argument("request_uuid", type=str)
@click.pass_obj
def search_resubmit(config, request_uuid):
    """
    Resubmit a data product search request

    \b
    REQUEST_UUID    the request unique identifier
    """
    # get request status
    try:
        click.echo("Retrieving query for request '%s' ..." % (request_uuid))
        url = pyaurorax.api.urls.data_products_request_url.format(request_uuid)
        status = pyaurorax.requests.get_status(url)
    except pyaurorax.AuroraXNotFoundException as e:
        click.echo("%s occurred: request ID not found" % (type(e).__name__))
        sys.exit(1)
    except pyaurorax.AuroraXException as e:
        click.echo("%s occurred: %s" % (type(e).__name__, e.args[0]))
        sys.exit(1)

    # set the query to use for resubmission
    if ("query" not in status["search_request"]):
        click.echo("Error resubmitting: missing query from original request ID")
        sys.exit(1)
    q = status["search_request"]["query"]

    # create search object
    click.echo("Preparing new search ...")
    s = __create_search_object_from_query(q)

    # submit search
    click.echo("Submitting new search ...")
    s.execute()

    # output new request ID
    click.echo("Request has been resubmitted, new request ID is %s" % (s.request_id))


@data_products_group.command("search_template",
                             short_help="Output template for a data product search request")
@click.option("--outfile", type=str, help="Save template to a file")
@click.option("--indent", type=int, default=2, show_default=True,
              help="Indentation to use when outputting template")
@click.pass_obj
def search_template(config, outfile, indent):
    """
    Output template for a data product search request
    """
    if (outfile is not None):
        with open(outfile, 'w', encoding="utf-8") as fp:
            json.dump(DATA_PRODUCTS_SEARCH_TEMPLATE, fp, indent=indent)
        click.echo("Saved template to %s" % (outfile))
    else:
        click.echo(json.dumps(DATA_PRODUCTS_SEARCH_TEMPLATE, indent=indent))


@data_products_group.command("search",
                             short_help="Perform a data product search request")
@click.argument("infile", type=str)
@click.option("--poll-interval",
              default=pyaurorax.requests.STANDARD_POLLING_SLEEP_TIME,
              show_default=True,
              help="Polling interval when waiting for data (seconds)")
@click.option("--outfile", type=str, help="Output file to save data to (a .json file)")
@click.option("--output-to-terminal", type=click.Choice(["dict", "objects"]),
              help="Output data to terminal in a certain format (instead of to file)")
@click.option("--indent", type=int, default=2, show_default=True,
              help="Indentation when saving data to file")
@click.option("--minify", is_flag=True, help="Minify the JSON data saved to file")
@click.option("--quiet", is_flag=True, help="Quiet output")
@click.pass_obj
def search(config, infile, poll_interval, outfile, output_to_terminal, indent, minify, quiet):
    """
    Perform a data product search request

    \b
    INFILE      input file with query (must be a JSON)
    """
    # check that infile exists
    if not (os.path.exists(infile)):
        click.echo("Error: infile doesn't exist (%s" % (infile))
        sys.exit(1)

    # read in infile
    if (quiet is False):
        click.echo("[%s] Reading in query file ..." % (datetime.datetime.now()))
    with open(infile, 'r', encoding="utf-8") as fp:
        q = json.load(fp)

    # set search params
    if (quiet is False):
        click.echo("[%s] Preparing search ..." % (datetime.datetime.now()))
    start = parse(q["start"], ignoretz=True)
    end = parse(q["end"], ignoretz=True)
    data_product_types = None if "data_product_type_filters" not in q else q["data_product_type_filters"]
    programs = None if "programs" not in q["data_sources"] else q["data_sources"]["programs"]
    platforms = None if "platforms" not in q["data_sources"] else q["data_sources"]["platforms"]
    instrument_types = None if "instrument_types" not in q["data_sources"] else q["data_sources"]["instrument_types"]
    metadata_filters = None
    metadata_filters_logical_operator = None
    if ("data_product_metadata_filters" in q["data_sources"]):
        if ("expressions" in q["data_sources"]["data_product_metadata_filters"]):
            metadata_filters = q["data_sources"]["data_product_metadata_filters"]["expressions"]
        if ("logical_operator" in q["data_sources"]["data_product_metadata_filters"]):
            metadata_filters_logical_operator = q["data_sources"]["data_product_metadata_filters"]["logical_operator"]
    verbose_search = True if quiet is False else False  # pylint: disable=simplifiable-if-expression

    # start search
    s = pyaurorax.data_products.search(start,
                                       end,
                                       programs=programs,
                                       platforms=platforms,
                                       instrument_types=instrument_types,
                                       data_product_types=data_product_types,
                                       metadata_filters=metadata_filters,
                                       metadata_filters_logical_operator=metadata_filters_logical_operator,
                                       poll_interval=poll_interval,
                                       verbose=verbose_search,
                                       return_immediately=True)

    # wait for data
    s.wait(poll_interval=poll_interval, verbose=verbose_search)

    # search has finished, save results to a file or output to terminal
    get_search_data("data_products",
                    s.request_id,
                    outfile,
                    output_to_terminal,
                    indent,
                    minify,
                    show_times=True,
                    search_obj=s)


@data_products_group.command("describe",
                             short_help="Describe a data product search request")
@click.argument("infile", type=str)
@click.pass_obj
def describe(config, infile):
    """
    Describe a data product search request using
    "SQL-like" syntax

    \b
    INFILE      input file with query (must be a JSON)
    """
    # check that infile exists
    if not (os.path.exists(infile)):
        click.echo("Error: infile doesn't exist (%s" % (infile))
        sys.exit(1)

    # read in infile
    with open(infile, 'r', encoding="utf-8") as fp:
        q = json.load(fp)

    # create search object
    s = __create_search_object_from_query(q)

    # describe the search
    d = pyaurorax.data_products.describe(s)

    # output
    click.echo(d)



"""
Helper functions when interacting with the API
"""

from typing import Dict
from .classes.urls import URLs
from ..api import DEFAULT_BASE_URL

# pdoc init
__pdoc__: Dict = {}

# private dynamic globals
__api_key: str = ""

# create instance of URLs that will be used throughout the application
urls = URLs()


def get_api_key() -> str:
    """
    Returns the currently set API key for the module

    Returns:
        current API key
    """
    return __api_key


def authenticate(api_key: str) -> None:
    """
    Set authentication values for use with subsequent queries

    Args:
        api_key: an AuroraX API key string
    """
    global __api_key
    __api_key = api_key


def set_base_url(url: str) -> None:
    """
    Change the base URL for the API (ie. change to the staging
    system or local server)

    Args:
        url: the new base url string (ie. 'https://api.staging.aurorax.space')
    """
    urls.base_url = url


def get_base_url() -> str:
    """
    Returns the current base URL for the API

    Returns:
        current base URL
    """
    return urls.base_url


def reset_base_url() -> None:
    """
    Set the base URL for the API back to the default
    """
    urls.base_url = DEFAULT_BASE_URL



"""
Interact with Swarm-Aurora using conjunction searches from AuroraX
"""

# function and class imports
from .tools import (get_url,
                    open_in_browser,
                    create_custom_import_file)

# pdoc imports and exports
from .tools import __pdoc__ as __tools_pdoc__
__pdoc__ = __tools_pdoc__
__all__ = [
    "get_url",
    "open_in_browser",
    "create_custom_import_file",
]



"""
The util module provides helper methods such as converting
arbitrary geographic locations to North/South B-trace geographic
locations.

Note that all functions and classes from submodules are all imported
at this level of the util module. They can be referenced from
here instead of digging in deeper to the submodules.
"""

# function and class imports
from .calculate_btrace import (ground_geo_to_nbtrace,
                               ground_geo_to_sbtrace)

# pdoc imports and exports
from .calculate_btrace import __pdoc__ as __btrace_pdoc__
__pdoc__ = __btrace_pdoc__
__all__ = [
    "ground_geo_to_nbtrace",
    "ground_geo_to_sbtrace",
]



"""
This class provides the URL endpoints for different AuroraX
API requests. It is contained in a special class so that we
can use different base URLs if desired.
"""

from typing import Dict
from ...api import DEFAULT_BASE_URL

# pdoc init
__pdoc__: Dict = {}


class URLs:
    __DEFAULT_URL_DATA_SOURCES = "/api/v1/data_sources"
    __DEFAULT_URL_DATA_SOURCES_SEARCH = "/api/v1/data_sources/search"
    __DEFAULT_URL_EPHEMERIS_AVAILABILITY = "/api/v1/availability/ephemeris"
    __DEFAULT_URL_EPHEMERIS_UPLOAD = "/api/v1/data_sources/{}/ephemeris"
    __DEFAULT_URL_EPHEMERIS_SEARCH = "/api/v1/ephemeris/search"
    __DEFAULT_URL_EPHEMERIS_REQUEST = "/api/v1/ephemeris/requests/{}"
    __DEFAULT_URL_DATA_PRODUCTS_AVAILABILITY = "/api/v1/availability/data_products"
    __DEFAULT_URL_DATA_PRODUCTS_UPLOAD = "/api/v1/data_sources/{}/data_products"
    __DEFAULT_URL_DATA_PRODUCTS_SEARCH = "/api/v1/data_products/search"
    __DEFAULT_URL_DATA_PRODUCTS_REQUEST = "/api/v1/data_products/requests/{}"
    __DEFAULT_URL_CONJUNCTION_SEARCH = "/api/v1/conjunctions/search"
    __DEFAULT_URL_CONJUNCTION_REQUEST = "/api/v1/conjunctions/requests/{}"
    __DEFAULT_URL_DESCRIBE_CONJUNCTION_QUERY = "/api/v1/utils/describe/query/conjunction"
    __DEFAULT_URL_DESCRIBE_DATA_PRODUCTS_QUERY = "/api/v1/utils/describe/query/data_products"
    __DEFAULT_URL_DESCRIBE_EPHEMERIS_QUERY = "/api/v1/utils/describe/query/ephemeris"

    def __init__(self, base_url: str = DEFAULT_BASE_URL) -> None:
        self.__base = base_url
        self.__data_sources = self.__DEFAULT_URL_DATA_SOURCES
        self.__data_sources_search = self.__DEFAULT_URL_DATA_SOURCES_SEARCH
        self.__ephemeris_availability = self.__DEFAULT_URL_EPHEMERIS_AVAILABILITY
        self.__ephemeris_search = self.__DEFAULT_URL_EPHEMERIS_SEARCH
        self.__ephemeris_upload = self.__DEFAULT_URL_EPHEMERIS_UPLOAD
        self.__ephemeris_request = self.__DEFAULT_URL_EPHEMERIS_REQUEST
        self.__data_products_availability = self.__DEFAULT_URL_DATA_PRODUCTS_AVAILABILITY
        self.__data_products_search = self.__DEFAULT_URL_DATA_PRODUCTS_SEARCH
        self.__data_products_upload = self.__DEFAULT_URL_DATA_PRODUCTS_UPLOAD
        self.__data_products_request = self.__DEFAULT_URL_DATA_PRODUCTS_REQUEST
        self.__conjunction_search = self.__DEFAULT_URL_CONJUNCTION_SEARCH
        self.__conjunction_request = self.__DEFAULT_URL_CONJUNCTION_REQUEST
        self.__describe_conjunction_query = self.__DEFAULT_URL_DESCRIBE_CONJUNCTION_QUERY
        self.__describe_data_products_query = self.__DEFAULT_URL_DESCRIBE_DATA_PRODUCTS_QUERY
        self.__describe_ephemeris_query = self.__DEFAULT_URL_DESCRIBE_EPHEMERIS_QUERY

    @property
    def base_url(self) -> str:
        return self.__base

    @base_url.setter
    def base_url(self, value: str) -> None:
        self.__base = value

    # data sources
    # -------------------
    @property
    def data_sources_url(self) -> str:
        return f"{self.__base}{self.__data_sources}"

    @property
    def data_sources_search_url(self) -> str:
        return f"{self.__base}{self.__data_sources_search}"

    # availability
    # -------------------
    @property
    def ephemeris_availability_url(self) -> str:
        return f"{self.__base}{self.__ephemeris_availability}"

    @property
    def data_products_availability_url(self) -> str:
        return f"{self.__base}{self.__data_products_availability}"

    # ephemeris
    # -------------------
    @property
    def ephemeris_search_url(self) -> str:
        return f"{self.__base}{self.__ephemeris_search}"

    @property
    def ephemeris_upload_url(self) -> str:
        return f"{self.__base}{self.__ephemeris_upload}"

    @property
    def ephemeris_request_url(self) -> str:
        return f"{self.__base}{self.__ephemeris_request}"

    # data products
    # -------------------
    @property
    def data_products_search_url(self) -> str:
        return f"{self.__base}{self.__data_products_search}"

    @property
    def data_products_upload_url(self) -> str:
        return f"{self.__base}{self.__data_products_upload}"

    @property
    def data_products_request_url(self) -> str:
        return f"{self.__base}{self.__data_products_request}"

    # conjunctions
    # -------------------
    @property
    def conjunction_search_url(self) -> str:
        return f"{self.__base}{self.__conjunction_search}"

    @property
    def conjunction_request_url(self) -> str:
        return f"{self.__base}{self.__conjunction_request}"

    # describe
    # -------------------
    @property
    def describe_conjunction_query(self) -> str:
        return f"{self.__base}{self.__describe_conjunction_query}"

    @property
    def describe_data_products_query(self) -> str:
        return f"{self.__base}{self.__describe_data_products_query}"

    @property
    def describe_ephemeris_query(self) -> str:
        return f"{self.__base}{self.__describe_ephemeris_query}"



"""
The conjunction module is used for finding conjunctions between
groupings of data sources.

Note that all functions and classes from submodules are all imported
at this level of the conjunctions module. They can be referenced from
here instead of digging in deeper to the submodules.
"""

# conjunction type - north b-trace
CONJUNCTION_TYPE_NBTRACE: str = "nbtrace"
"""
Conjunction search 'conjunction_type' category for
finding conjunctions using the north B-trace data
"""

# conjunction type - south b-trace
CONJUNCTION_TYPE_SBTRACE: str = "sbtrace"
"""
Conjunction search 'conjunction_type' category for
finding conjunctions using the south B-trace data
"""

# function and class imports
from .conjunctions import (search,
                           search_async,
                           describe,
                           get_request_url)
from .swarmaurora import __all__ as swarmaurora_all
from .classes.conjunction import Conjunction
from .classes.search import Search

# pdoc imports and exports
from .conjunctions import __pdoc__ as __conjunctions_pdoc__
from .classes.conjunction import __pdoc__ as __classes_conjunctions_pdoc__
from .classes.search import __pdoc__ as __classes_search_pdoc__
__pdoc__ = __conjunctions_pdoc__
__pdoc__ = dict(__pdoc__, **__classes_conjunctions_pdoc__)
__pdoc__ = dict(__pdoc__, **__classes_search_pdoc__)
__all__ = [
    "CONJUNCTION_TYPE_NBTRACE",
    "CONJUNCTION_TYPE_SBTRACE",
    "search",
    "search_async",
    "describe",
    "get_request_url",
    "Conjunction",
    "Search",
    "swarmaurora_all",
]
